<?php

/**
 * The OpenEDUHttpRequest class.
 */
class OpenEDUHttpRequest {

  public $token = NULL;
  public $headers = array();
  public $method = NULL;
  public $data = array();
  public $request = NULL;
  public $endpoint = NULL;
  public $timeout = 30;
  public $accept = 'application/json';
  public $content_type = 'application/json';

  /**
   * Constructor.
   *
   * @param string $endpoint
   *  Endpoint URL.
   * @param array $data
   *  Data to be submitted against endpoint.
   * @param string $action
   *  Action being performed.
   * @param string $token
   *  Token used during validation.
   */
  public function __construct($endpoint = NULL, $data = array(), $action = NULL, $token = NULL) {
    $this->setHeaders($token);
    $this->setEndpoint($endpoint);
    $this->setData($data);
    $this->setMethod($action);
  }

  /**
   * Set token
   *
   * @param string $value
   *  Token value.
   */
  public function setToken($value) {
    $this->token = $value;
  }

  /**
   * Add custom info into header array.
   *
   * @param array $params
   *  Information being added.
   */
  public function addHeaders(array $params) {
    $this->headers = array_merge($this->headers, $params);
  }

  /**
   * Set required header information to access endpoint.
   */
  public function setHeaders($token = NULL) {
    if (isset($token)) {
      $this->setToken($token);
    }

    $this->headers = array(
      'Accept' => $this->accept,
      'Content-Type' => $this->content_type,
      'Openedu-Token' => $this->token,
    );
  }

  /**
   * Set Accept parameter.
   *
   * @param $accept
   *  Accept to be used.
   */
  public function setAccept($accept) {
    $this->accept = $accept;
  }

  /**
   * Set Content-Type parameter.
   *
   * @param $content_type
   *  Content-Type to be used.
   */
  public function setContentType($content_type) {
    $this->content_type = $content_type;
  }

  /**
   * Set method to be used during request.
   *
   * @param string $method
   *  Method to be used.
   */
  public function setMethod($method) {
    $this->method = $this->defineMethod($method);
  }

  /**
   * Set data.
   *
   * @param array $data
   *  Data array.
   */
  public function setData($data = array()) {
    $this->data = (is_array($data) ? json_encode($data) : $data);
  }

  /**
   * Set endpoint URL.
   *
   * @param string $endpoint
   *  Endpoint URL.
   */
  public function setEndpoint($endpoint = NULL) {
    $this->endpoint = $endpoint;
  }

  /**
   * Set the maximum number of seconds the function call may take.
   *
   * @param int $timeout
   *  Number of seconds.
   */
  public function setTimeout($timeout = 30) {
    $this->timeout = $timeout;
  }

  /**
   * Defines http method based on action that will be taken.
   *
   * @param string $action
   *  Action being performed.
   *
   * @return string
   */
  public function defineMethod($action = NULL) {
    $methods = array(
      'retrieve' => 'GET',
      'update' => 'PUT',
      'create' => 'POST',
      'delete' => 'DELETE'
    );

    return (isset($methods[$action]) ? $methods[$action] : 'GET');
  }

  /**
   * Execute request against endpoint.
   *
   * @return $this
   */
  public function execute() {
    // Submit request.
    $this->request = drupal_http_request($this->endpoint, array(
      'headers' => $this->headers,
      'method' => $this->method,
      'data' => $this->data,
      'timeout' => $this->timeout
    ));

    return $this;
  }
}