<?php

/**
 * @file openedu_notifications.inc
 */
abstract class OpenEDUNotification {
  const TYPE = 'base';

  function __construct($vars = NULL) {
    if (is_object($vars)) {
      foreach ($vars as $key => $value) {
        $this->{$key} = $value;
      }
    }

    $this->message = '';
    $this->timestamp = REQUEST_TIME;
  }

  function type() {
    return self::TYPE;
  }

  /**
   * Return the default data for the queue.
   *
   * @return array
   */
  static function get_default_data() {
    global $user;
    $client_var = openedu_core_contextualized_variable_get(OPENEDU_NOTIFICATIONS_CLIENT_VAR_GENERAL, FALSE);

    return array(
      'status' => 1,
      // Notification status.
      'class' => '',
      // Handler class.
      'uid' => $user->uid,
      // User uid.
      'user_uuid' => $user->uuid,
      // User UUID.
      'entity_id' => '',
      // Entity id.
      'entity_type' => '',
      // Entity type.
      'entity_state' => '',
      // Entity state machine.
      'entity_link' => '',
      // Entity link.
      'entity_data' => '',
      // Entity data.
      'client_uuid' => $client_var['uuid'],
      // Client UUID.
      'action' => '',
      // Action being performed against entity.
      'message' => '',
      // Custom message.
      'created' => REQUEST_TIME,
      // When notification was created.
    );
  }

  /**
   * Find out all the notifications related to a given entity object.
   *
   * @param $entity_type
   * @param $entity_id
   * @return array
   */
  static function get_all_groups($entity_type, $entity_id) {
    $groups = array();
    $classes = openedu_notifications_client_get_all_classes();
    foreach ($classes as $class_name => $class_config) {
      $groups[] = $class_name::get_notification_groups($entity_type, $entity_id);
    }
    return array_filter($groups);
  }

  /**
   * @return mixed
   */
  abstract function queue($client_data = array());

  /**
   * Find notifications for a given entity object.
   *
   * @param $entity_type
   * @param $entity_id
   * @return mixed
   */
   static function get_notification_group($entity_type, $entity_id) {
     // This needs to be implemented for all inheriting classes.
   }
}

/**
 * Class RoleNotification
 * @package OpenEDU
 */
class OpenEDURoleNotification extends OpenEDUNotification {
  const TYPE = 'role';

  function __construct($vars = NULL) {
    parent::__construct($vars);
  }

  /**
   * @todo: Implement.
   *
   * @return bool
   */
  public function queue($client_data = array()) {
    return FALSE;
  }

  /**
   * Find notifications for a given entity object.
   * @todo: Convert the procedural approach for role notifications.
   *
   * @param $entity_type
   * @param $entity_id
   * @return mixed
   */
  static function get_notification_group($entity_type, $entity_id) {
    return FALSE;
  }
}

/**
 * Class UserNotification
 * @package OpenEDU
 */
class OpenEDUUserNotification extends OpenEDUNotification {
  const TYPE = 'user';

  function __construct($vars = NULL) {
    parent::__construct($vars);
  }

  /**
   * Insert this notificaiton into the queue.
   */
  public function queue($client_data = array()) {
    $action = empty($client_data['action']) ? 'default' : $client_data['action'];
    $default_data = self::get_default_data();
    // Entity URI.
    $entity_uri = entity_uri($this->entity_type, $client_data['entity_data']);
    $client_data = array_merge($client_data, array(
      'class' => __CLASS__,
      'uid' => $this->uid,
      'user_uuid' => $this->user_uuid,
      'entity_type' => openedu_notifications_create_entity_bundle_type($client_data['entity_data'], $this->entity_type),
      'entity_id' => $this->entity_id,
      'entity_link' => isset($entity_uri['path']) ? url($entity_uri['path'], array('absolute' => TRUE)) : 'N/A',
      'entity_data' => serialize($client_data['entity_data']),
    ));
    return openedu_notifications_client_queue_notifications(array_merge($default_data, $client_data), $action);
  }

  /**
   * Find notifications for a given entity object.
   *
   * @param $entity_type
   * @param $entity_id
   * @return mixed
   */
  static function get_notification_group($entity_type, $entity_id) {
    $group = new OpenEDUNotificationGroup();
    $subscriptions = OpenEDUUserSubscription::load_by_entity($entity_type, $entity_id);
    foreach ($subscriptions as $subscription) {
      $group->push(new OpenEDUUserNotification($subscription));
    }
    return $group;
  }
}

/**
 * Class NotificationGroup
 * @package OpenEDU
 */
class OpenEDUNotificationGroup {
  const QUEUE_TABLE = 'openedu_notifications_client_notifications_queue';

  // This variable will store the data to be sent via HTTP request.
  public $request;
  protected $notifications = array();

  function __construct($config = array()) {

  }

  public function count() {
    return count($this->notifications);
  }

  /**
   * Simple push method.
   *
   * @param $notification
   * @throws Exception
   */
  public function push($notification) {
    if (!is_object($notification)) {
      throw(new Exception(t('NotificationGroup->push() can only accept Notification objects.')));
    }
    array_push($this->notifications, $notification);
  }

  /**
   * Insert items into the queue.
   */
  public function queue($client_data = array()) {
    foreach ($this->notifications as $notification) {
      $notification->queue($client_data);
    }
  }

  /**
   * Not used at the moment.
   *
   * @return array
   */
  private function form_request_body() {
    return array();
  }
}

class OpenEDUUserSubscription {
  const DB_TABLE = 'openedu_notifications_client_user_subscriptions';

  public $uid;
  public $user_uuid;
  public $entity_type;
  public $entity_id;

  function __construct($uid, $user_uuid, $entity_type, $entity_id) {
    $this->uid = $uid;
    $this->user_uuid = $user_uuid;
    $this->entity_type = $entity_type;
    $this->entity_id = $entity_id;
  }

  /**
   * Whether or not this subscription already exists.
   *
   * @return bool
   */
  public function is_subscribed() {
    $count = db_select(self::DB_TABLE)
      ->condition('uid', $this->uid)
      ->condition('user_uuid', $this->user_uuid)
      ->condition('entity_type', $this->entity_type)
      ->condition('entity_id', $this->entity_id)
      ->countQuery()
      ->execute()
      ->fetchField();
    return empty($count) ? FALSE : TRUE;
  }

  /**
   * Write database record.
   */
  public function save() {
    $record = array(
      'uid' => $this->uid,
      'user_uuid' => $this->user_uuid,
      'entity_type' => $this->entity_type,
      'entity_id' => $this->entity_id,
    );
    $result = db_merge(self::DB_TABLE)
      ->key($record)
      ->fields($record)
      ->execute();
    return $result;
  }

  /**
   * Remove database record;
   */
  public function unsave() {
    $result = db_delete(self::DB_TABLE)
      ->condition('uid', $this->uid)
      ->condition('user_uuid', $this->user_uuid)
      ->condition('entity_type', $this->entity_type)
      ->condition('entity_id', $this->entity_id)
      ->execute();
    return $result;
  }

  /**
   * Get subscriptions related to a given entity object.
   *
   * @param $entity_type
   * @param $entity_id
   * @return array
   */
  public static function load_by_entity($entity_type, $entity_id) {
    $subscriptions = array();
    $results = db_select(self::DB_TABLE, 'subs')
      ->fields('subs')
      ->condition('entity_type', $entity_type)
      ->condition('entity_id', $entity_id)
      ->execute();
    foreach ($results as $row) {
      $subscriptions[] = new OpenEDUUserSubscription($row->uid, $row->user_uuid, $row->entity_type, $row->entity_id);
    }
    return $subscriptions;
  }

  /**
   * Get subscriptions of a given user.
   *
   * @param $user_uuid
   * @return array
   */
  public static function load_by_user($user_uuid) {
    $subscriptions = array();
    $results = db_select(self::DB_TABLE, 'subs')
      ->fields('subs')
      ->condition('user_uuid', $user_uuid)
      ->execute();
    foreach ($results as $row) {
      $subscriptions[] = new OpenEDUUserSubscription($row->uid, $row->user_uuid, $row->entity_type, $row->entity_id);
    }
    return $subscriptions;
  }
}
