<?php
/**
 * @file
 * OpenEDU Notifications notifications resource configuration.
 */

/**
 * Access callback for the server endpoint and allow users to retrieve all data
 * since passed token validation.
 *
 * @param string $op
 *  The operation that's going to be performed.
 *
 * @return bool
 *  Whether access is given or not.
 */
function openedu_notifications_server_notifications_resource_access($op) {
  switch ($op) {
    // Index, Create, Update, Delete calls.
    case 'index':
    case 'update':
    case 'delete':
    case 'create':
      $access = TRUE;
      break;

    default:
      $access = FALSE;
  }

  return $access;
}

/**
 * Callback for creating a notification.
 *
 * @param array $data
 *   An array of parameters used to create an notification pragmatically.
 *   Available keys: uid (int), client (string), type (string),
 *   message (string).
 *
 * @return object
 */
function openedu_notifications_server_notifications_resource_create($data) {
  // Set vars.
  $successful = array();
  $unsuccessful = array();

  // Set mandatory fields.
  $mandatory_fields = array(
    'uid',
    'user_uuid',
    'entity_id',
    'entity_link',
    'client_uuid',
  );

  // Check if data being sent is a multidimensional array.
  if (!is_array(current($data))) {
    return services_error('Data must be a multidimensional.', 406);
  }

  foreach ($data as $id => $item) {
    // Check if a few mandatory fields are present and try saving item.
    if (TRUE === openedu_core_check_mandatory_fields($mandatory_fields, $item) &&
      SAVED_NEW == openedu_notifications_server_notifications_write_record($item)) {
      $successful[] = $id;
    }
    else {
      $unsuccessful[] = $id;
    }
  }

  // If no successful items means something went wrong.
  if (empty($successful)) {
    return services_error('Error trying to create notification', 400);
  }

  // Otherwise return updated items to be updated on the other side.
  return array(
    'succeed' => $successful,
    'failed' => $unsuccessful
  );
}

/**
 * Callback for updating a notification.
 *
 * @param string $token
 *   A string containing the hash used to validate the client's identity.
 * @param array $data
 *   An array of parameters used to create an notification pragmatically.
 *   Available keys: uid (int), client (string), type (string),
 *   message (string).
 *
 * @return object
 */
function openedu_notifications_server_notifications_resource_update($token, $data) {
  // Set mandatory fields.
  $mandatory_fields = array(
    'uid',
    'entity_id',
    'entity_link',
    'client_uuid',
  );

  // Check if a few mandatory fields are present.
  if (FALSE === openedu_core_check_mandatory_fields($mandatory_fields, $data)) {
    return services_error('Missing attribute, ensure all mandatory fields are being sent', 406);
  }

  // Attempt to update record.
  $updated = openedu_notifications_server_notifications_update($data);

  // Check if update was executed.
  if (SAVED_UPDATED === $updated) {
    return array(
      'status' => 200,
      'result' => 'Record updated.'
    );
  }

  return services_error('Error updating record', 406);
}

/**
 * Callback for listing notifications stored on server.
 *
 * @param string $types
 *  Entity types to be retrieved.
 * @param string $groupby
 *  Group results by.
 * @param string $orderby
 *  Order by.
 * @param int $total
 *  Total number of notifications being retrieved.
 * @param int $status
 *  Notification status.
 * @param string $fields
 *  Fields been retrieved.
 *
 * @return string
 *  JSON string.
 */
function openedu_notifications_server_notifications_resource_index($types = '', $groupby, $orderby, $total, $status, $id, $eid, $fields, $user_uuid, $class) {
  // Load endpoint info.
  global $_openedu_authentication_endpoint_requester;

  return openedu_notifications_server_retrieve_notifications(array(
    'uuid' => isset($_openedu_authentication_endpoint_requester->uuid) ? $_openedu_authentication_endpoint_requester->uuid : FALSE,
    'types' => $types,
    'class' => isset($class) ? $class : '',
    'groupby' => isset($groupby) ? $groupby : FALSE,
    'user_uuid' => isset($user_uuid) ? $user_uuid : '',
    'orderby' => (isset($orderby) && strtolower($orderby) == 'asc') ? 'ASC' : 'DESC',
    'total' => (isset($total) && is_numeric($total)) ? $total : 500,
    'status' => (isset($status) && is_numeric($status)) ? $status : 1,
    'id' => isset($id) ? explode(',', $id) : FALSE,
    'eid' => isset($eid) ? explode(',', $eid) : FALSE,
    'fields' => isset($fields) ? explode(',', $fields) : array(
      'nid',
      'status',
      'class',
      'user_uuid',
      'uid',
      'entity_id',
      'entity_type',
      'entity_state',
      'entity_link',
      'entity_data',
      'client_uuid',
      'action',
      'message',
      'created'
    )
  ));
}
